@@ -15,8 +15,8 @@ mod xorurl_media_types;
 
 pub use errors::{Error, Result};
 use multibase::{decode as base_decode, encode as base_encode, Base};
-use safe_network::types::{BytesAddress, DataAddress, RegisterAddress, SafeKeyAddress, Scope};
 use serde::{Deserialize, Serialize};
+use sn_interface::types::{BytesAddress, DataAddress, RegisterAddress, SafeKeyAddress, Scope};
 use std::fmt;
 use tracing::{info, trace, warn};
 use url::Url;
@@ -1207,7 +1207,7 @@ mod tests {
     use super::*;
     use color_eyre::{eyre::bail, eyre::eyre, Result};
     use rand::Rng;
-    use safe_network::types::{register::EntryHash, BytesAddress};
+    use sn_interface::types::{register::EntryHash, BytesAddress};
 
     macro_rules! verify_expected_result {
             ($result:expr, $pattern:pat $(if $cond:expr)?) => {
@@ -1854,7 +1854,7 @@ mod tests {
             "safe://hnyydy iixsfrqix9aoqg97jebuzc6748uc8rykhdd5hjrtg5o4xso9jmggbqh",
         )
         .expect_err(msg);
-        println!("{:#?}", result);
+
         match result {
             Error::InvalidInput(e) => assert!(e.contains("The URL cannot contain whitespace")),
             _ => bail!(wrong_err),
